@@ -156,7 +156,10 @@ def _resolve_dfs(raw_io):
     referral = dfs_request(raw_io.fd.tree_connect, raw_path[1:])
     client_config.cache_referral(referral)
     info = client_config.lookup_referral([p for p in raw_path.split("\\") if p])
-    connection_kwargs = getattr(raw_io, '_%s__kwargs' % type(raw_io).__name__, {})
+    if not info:
+        raise ObjectPathNotFound()
+
+    connection_kwargs = getattr(raw_io, '_%s__kwargs' % SMBRawIO.__name__, {})
 
     for target in info:
         new_path = raw_path.replace(info.dfs_path, target.target_path, 1)
