@@ -5,6 +5,7 @@
 import atexit
 import logging
 import ntpath
+import typing
 import uuid
 import warnings
 
@@ -27,6 +28,7 @@ from smbprotocol.dfs import (
 from smbprotocol.exceptions import (
     BadNetworkName,
     InvalidParameter,
+    ObjectPathNotFound,
 )
 
 from smbprotocol.ioctl import (
@@ -141,17 +143,20 @@ class ClientConfig(object, metaclass=_ConfigSingleton):
             self._domain_cache.append(DomainEntry(domain_referral))
 
     def cache_referral(self, referral):
-        self._referral_cache.append(ReferralEntry(referral))
+        if referral['number_of_referrals'].get_value() > 0:
+            self._referral_cache.append(ReferralEntry(referral))
 
     def lookup_domain(self, domain_name):  # type: (str) -> Optional[DomainEntry]
+        # TODO: Check domain referral expiry and resend request if expired
         for domain in self._domain_cache:
             if domain.domain_name.lower() == ("\\" + domain_name.lower()):
                 return domain
 
-    def lookup_referral(self, path_components):  # type: (List[str]) -> Optional[ReferralEntry]
+    def lookup_referral(self, path_components: typing.List[str]) -> typing.Optional[ReferralEntry]:
         """ Checks if the path exists in the DFS referral cache. """
         # A lookup in ReferralCache involves searching for an entry with DFSPathPrefix that is a complete prefix of the
         # path being looked up.
+        self._clear_expired_cache()
         hits = []
         for referral in self._referral_cache:
             referral_path_components = [p for p in referral.dfs_path.split("\\") if p]
@@ -184,6 +189,9 @@ class ClientConfig(object, metaclass=_ConfigSingleton):
         if domain_controller:
             self.domain_controller = config['domain_controller']
 
+    def _clear_expired_cache(self) -> None:
+        self._referral_cache = [refferal for refferal in self._referral_cache if not refferal.is_expired]
+
 
 def dfs_request(tree, path):  # type: (TreeConnect, str) -> DFSReferralResponse
     """ Send a DFS Referral request to the IPC tree and return the referrals. """
@@ -292,6 +300,9 @@ def get_smb_tree(path, username=None, password=None, port=445, encrypt=None, con
             referral_response = dfs_request(ipc_tree, "\\%s\\%s" % (path_split[0], path_split[1]))
             client_config.cache_referral(referral_response)
             referral = client_config.lookup_referral(path_split)
+            if not referral:
+                raise ObjectPathNotFound()
+
             path = path.replace(referral.dfs_path, referral.target_hint.target_path, 1)
             path_split = [p for p in path.split("\\") if p]
 
@@ -315,6 +326,11 @@ def get_smb_tree(path, username=None, password=None, port=445, encrypt=None, con
             ipc_tree = get_smb_tree(ipc_path, **get_kwargs)[0]
             referral = dfs_request(ipc_tree, "\\%s\\%s" % (path_split[0], path_split[1]))
             client_config.cache_referral(referral)
+
+            # Sometimes a DFS referral may return 0 referrals, this needs to be checked here to avoid repeats.
+            if not client_config.lookup_referral(path_split):
+                raise ObjectPathNotFound()
+
             return get_smb_tree(path, **get_kwargs)
 
     file_path = ""
