@@ -2,9 +2,6 @@
 # Copyright: (c) 2019, Jordan Borean (@jborean93) <jborean93@gmail.com> and other contributors
 # MIT License (see LICENSE or https://opensource.org/licenses/MIT)
 
-from __future__ import absolute_import
-from __future__ import division
-
 import errno
 import ntpath
 import os
@@ -31,6 +28,7 @@ from smbclient._os import (
     stat as smbclient_stat,
     symlink,
     SMBDirEntry,
+    is_remote_path,
 )
 
 from smbclient.path import (
@@ -43,10 +41,6 @@ from smbprotocol import (
     MAX_PAYLOAD_SIZE,
 )
 
-from smbprotocol._text import (
-    to_native,
-)
-
 from smbprotocol.file_info import (
     FileAttributes,
     FileBasicInformation,
@@ -62,6 +56,37 @@ from smbprotocol.structure import (
 )
 
 
+def _basename(path):
+    """
+    Return the base name of pathname path.
+    This is the second element of the pair returned by passing path to the function split().
+    Note that the result of this function is different from the Unix basename program;
+    where basename for '/foo/bar/' returns 'bar', the basename() function returns an empty string ('').
+
+    :param path: The path to take the base of.
+    :return: The basename of the path.
+    """
+    if is_remote_path(path):
+        return ntpath.basename(path)
+    else:
+        return os.path.basename(path)
+
+
+def _join_local_or_remote_path(path, *paths):
+    """
+    Return the joined paths regardless of whether they are local or remote.
+    Currently, this only makes the decision based on the base path, and does not raise a ValueError if the additional
+    paths do not match.
+    :param path: The base path.
+    :param paths: The additional paths to append.
+    :return: Joined path.
+    """
+    if is_remote_path(path):
+        return ntpath.join(path, *paths)
+    else:
+        return os.path.join(path, *paths)
+
+
 def copy(src, dst, follow_symlinks=True, **kwargs):
     """
     Copies the file src to the file or directory dst. If dst specified a directory, the file will be copied into dst
@@ -123,14 +148,8 @@ def copyfile(src, dst, follow_symlinks=True, **kwargs):
     :param kwargs: Common arguments used to build the SMB Session for any UNC paths.
     :return: The dst path.
     """
-    def wrap_not_implemented(function_name):
-        def raise_not_implemented(*args, **kwargs):
-            raise NotImplementedError("%s is unavailable on this platform as a local operation" % function_name)
-
-        return raise_not_implemented
-
     norm_src = ntpath.normpath(src)
-    if norm_src.startswith('\\\\'):
+    if is_remote_path(norm_src):
         src_root = ntpath.splitdrive(norm_src)[0]
         islink_func = islink
         readlink_func = readlink
@@ -140,14 +159,13 @@ def copyfile(src, dst, follow_symlinks=True, **kwargs):
     else:
         src_root = None
         islink_func = os.path.islink
-        # readlink and symlink are not available on Windows on Python 2.
-        readlink_func = getattr(os, 'readlink', wrap_not_implemented('readlink'))
-        symlink_func = getattr(os, 'symlink', wrap_not_implemented('symlink'))
+        readlink_func = os.readlink
+        symlink_func = os.symlink
         src_open = open
         src_kwargs = {}
 
     norm_dst = ntpath.normpath(dst)
-    if norm_dst.startswith('\\\\'):
+    if is_remote_path(norm_dst):
         dst_root = ntpath.splitdrive(norm_dst)[0]
         dst_open = open_file
         dst_kwargs = kwargs
@@ -180,7 +198,7 @@ def copyfile(src, dst, follow_symlinks=True, **kwargs):
                 raise
 
         if is_same:
-            raise shutil.Error(to_native("'%s' and '%s' are the same file, cannot copy" % (src, dst)))
+            raise shutil.Error("'%s' and '%s' are the same file, cannot copy" % (src, dst))
 
         smbclient_copyfile(src, dst, **kwargs)
         return dst
@@ -216,11 +234,11 @@ def copymode(src, dst, follow_symlinks=True, **kwargs):
     src_mode = stat.S_IMODE(_get_file_stat(src, follow_symlinks, **kwargs).st_mode)
 
     norm_dst = ntpath.normpath(dst)
-    if norm_dst.startswith('\\\\'):
+    if is_remote_path(norm_dst):
         read_only = not (src_mode & stat.S_IWRITE == stat.S_IWRITE and src_mode & stat.S_IREAD == stat.S_IREAD)
         _set_file_basic_info(dst, follow_symlinks, read_only=read_only, **kwargs)
     else:
-        _local_chmod(dst, src_mode, follow_symlinks)
+        os.chmod(dst, src_mode, follow_symlinks=follow_symlinks)
 
 
 def copystat(src, dst, follow_symlinks=True, **kwargs):
@@ -239,25 +257,16 @@ def copystat(src, dst, follow_symlinks=True, **kwargs):
     src_stat = _get_file_stat(src, follow_symlinks, **kwargs)
     src_mode = stat.S_IMODE(src_stat.st_mode)
 
-    # *_ns was only added in Python 3, fallback to a manual calculation from seconds since EPOCH.
-    atime_ns = getattr(src_stat, 'st_atime_ns', src_stat.st_atime * 1000000000)
-    mtime_ns = getattr(src_stat, 'st_mtime_ns', src_stat.st_mtime * 1000000000)
+    atime_ns = src_stat.st_atime_ns
+    mtime_ns = src_stat.st_mtime_ns
 
     norm_dst = ntpath.normpath(dst)
-    if norm_dst.startswith('\\\\'):
+    if is_remote_path(norm_dst):
         read_only = not (src_mode & stat.S_IWRITE == stat.S_IWRITE and src_mode & stat.S_IREAD == stat.S_IREAD)
         _set_file_basic_info(dst, follow_symlinks, read_only=read_only, atime_ns=atime_ns, mtime_ns=mtime_ns, **kwargs)
     else:
-        if not follow_symlinks and sys.version_info[0] < 3:
-            # Python 2 always follows symlinks and does not have a kwarg to override, we can only just fail here.
-            raise NotImplementedError("utime: follow_symlinks unavailable on this platform")
-
-        _local_chmod(dst, src_mode, follow_symlinks)
-
-        if sys.version_info[0] < 3:
-            os.utime(dst, (atime_ns / 1000000000, mtime_ns / 1000000000))
-        else:
-            os.utime(dst, ns=(atime_ns, mtime_ns), follow_symlinks=follow_symlinks)
+        os.chmod(dst, src_mode, follow_symlinks=follow_symlinks)
+        os.utime(dst, ns=(atime_ns, mtime_ns), follow_symlinks=follow_symlinks)
 
 
 def copytree(src, dst, symlinks=False, ignore=None, copy_function=copy2, ignore_dangling_symlinks=False,
@@ -315,8 +324,8 @@ def copytree(src, dst, symlinks=False, ignore=None, copy_function=copy2, ignore_
         if dir_entry.name in ignored:
             continue
 
-        src_path = ntpath.join(src, dir_entry.name)
-        dst_path = ntpath.join(dst, dir_entry.name)
+        src_path = _join_local_or_remote_path(src, dir_entry.name)
+        dst_path = _join_local_or_remote_path(dst, dir_entry.name)
 
         try:
             if dir_entry.is_symlink():
@@ -407,19 +416,19 @@ def rmtree(path, ignore_errors=False, onerror=None, **kwargs):
         if dir_entry.is_symlink() and \
                 dir_entry.stat(follow_symlinks=False).st_file_attributes & FileAttributes.FILE_ATTRIBUTE_DIRECTORY:
             try:
-                rmdir(dir_entry.path)
+                rmdir(dir_entry.path, **kwargs)
             except OSError:
                 onerror(rmdir, dir_entry.path, sys.exc_info())
         elif dir_entry.is_dir():
-            rmtree(dir_entry.path, ignore_errors, onerror)
+            rmtree(dir_entry.path, ignore_errors, onerror, **kwargs)
         else:
             try:
-                remove(dir_entry.path)
+                remove(dir_entry.path, **kwargs)
             except OSError:
                 onerror(remove, dir_entry.path, sys.exc_info())
 
     try:
-        rmdir(path)
+        rmdir(path, **kwargs)
     except OSError:
         onerror(rmdir, path, sys.exc_info())
 
@@ -427,10 +436,8 @@ def rmtree(path, ignore_errors=False, onerror=None, **kwargs):
 def _copy(src, dst, follow_symlinks, copy_meta_func, **kwargs):
     # Need to check if dst is a UNC path before checking if it's a dir in smbclient.path before checking to see if it's
     # a local directory. If either one is a dir, join the filename of src onto dst.
-    if ntpath.normpath(dst).startswith('\\\\') and isdir(dst, **kwargs):
-        dst = ntpath.join(dst, ntpath.basename(src))
-    elif os.path.isdir(dst):
-        dst = os.path.join(dst, os.path.basename(src))
+    if (is_remote_path(ntpath.normpath(dst)) and isdir(dst, **kwargs)) or os.path.isdir(dst):
+        dst = _join_local_or_remote_path(dst, _basename(src))
 
     copyfile(src, dst, follow_symlinks=follow_symlinks)
     copy_meta_func(src, dst, follow_symlinks=follow_symlinks)
@@ -448,18 +455,6 @@ def _get_file_stat(path, follow_symlinks=True, **kwargs):
             return os.lstat(path)
 
 
-def _local_chmod(path, mode, follow_symlinks):
-    if sys.version_info[0] < 3:
-        if not follow_symlinks:
-            if not hasattr(os, 'lchmod'):
-                raise NotImplementedError("chmod: follow_symlinks unavailable on this platform")
-            os.lchmod(path, mode)
-        else:
-            os.chmod(path, mode)
-    else:
-        os.chmod(path, mode, follow_symlinks=follow_symlinks)
-
-
 def _set_file_basic_info(file_path, follow_symlinks, read_only=None, atime_ns=None, mtime_ns=None, **kwargs):
     co = 0 if follow_symlinks else CreateOptions.FILE_OPEN_REPARSE_POINT
 
