@@ -57,6 +57,7 @@ class NtStatus(object):
     not an exhaustive list but common values that are returned.
     """
     STATUS_SUCCESS = 0x00000000
+    STATUS_UNSUCCESSFUL = 0xC0000001
     STATUS_NETWORK_NAME_DELETED = 0xC00000C9
     STATUS_PENDING = 0x00000103
     STATUS_NOTIFY_CLEANUP = 0x0000010B
@@ -67,6 +68,7 @@ class NtStatus(object):
     STATUS_INVALID_EA_NAME = 0x80000013
     STATUS_EA_LIST_INCONSISTENT = 0x80000014
     STATUS_STOPPED_ON_SYMLINK = 0x8000002D
+    STATUS_INVALID_INFO_CLASS = 0xC0000003
     STATUS_INFO_LENGTH_MISMATCH = 0xC0000004
     STATUS_INVALID_PARAMETER = 0xC000000D
     STATUS_NO_SUCH_FILE = 0xC000000F
@@ -86,15 +88,18 @@ class NtStatus(object):
     STATUS_NONEXISTENT_EA_ENTRY = 0xC0000051
     STATUS_NO_EAS_ON_FILE = 0xC0000052
     STATUS_EA_CORRUPT_ERROR = 0xC0000053
+    STATUS_DELETE_PENDING = 0xC0000056
     STATUS_PRIVILEGE_NOT_HELD = 0xC0000061
     STATUS_WRONG_PASSWORD = 0xC000006A
     STATUS_LOGON_FAILURE = 0xC000006D
     STATUS_PASSWORD_EXPIRED = 0xC0000071
+    STATUS_NONE_MAPPED = 0xC0000073
     STATUS_INSUFFICIENT_RESOURCES = 0xC000009A
     STATUS_PIPE_NOT_AVAILABLE = 0xC00000AC
     STATUS_PIPE_BUSY = 0xC00000AE
     STATUS_PIPE_DISCONNECTED = 0xC00000B0
     STATUS_PIPE_CLOSING = 0xC00000B1
+    STATUS_IO_TIMEOUT = 0xC00000B5
     STATUS_FILE_IS_A_DIRECTORY = 0xC00000BA
     STATUS_NOT_SUPPORTED = 0xC00000BB
     STATUS_BAD_NETWORK_NAME = 0xC00000CC
@@ -107,7 +112,9 @@ class NtStatus(object):
     STATUS_CANNOT_DELETE = 0xC0000121
     STATUS_FILE_CLOSED = 0xC0000128
     STATUS_PIPE_BROKEN = 0xC000014B
+    STATUS_FS_DRIVER_REQUIRED = 0xC000019C
     STATUS_USER_SESSION_DELETED = 0xC0000203
+    STATUS_INSUFF_SERVER_RESOURCES = 0xC0000205
     STATUS_NOT_FOUND = 0xC0000225
     STATUS_PATH_NOT_COVERED = 0xC0000257
     STATUS_DFS_UNAVAILABLE = 0xC000026D
