@@ -459,7 +459,8 @@ class SMB2CreateResponse(Structure):
             ('end_of_file', IntField(size=8)),
             ('file_attributes', FlagField(
                 size=4,
-                flag_type=FileAttributes
+                flag_type=FileAttributes,
+                flag_strict=False,
             )),
             ('reserved2', IntField(size=4)),
             ('file_id', BytesField(size=16)),
@@ -554,7 +555,8 @@ class SMB2CloseResponse(Structure):
             ('end_of_file', IntField(size=8)),
             ('file_attributes', FlagField(
                 size=4,
-                flag_type=FileAttributes
+                flag_type=FileAttributes,
+                flag_strict=False,
             ))
         ])
         super(SMB2CloseResponse, self).__init__()
@@ -1035,9 +1037,9 @@ class SMB2SetInfoRequest(Structure):
                 default=96
             )),
             ('reserved', IntField(size=2)),
-            ('additional_information', EnumField(
+            ('additional_information', FlagField(
                 size=4,
-                enum_type=InfoAdditionalInformation,
+                flag_type=InfoAdditionalInformation,
             )),
             ('file_id', BytesField(size=16)),
             ('buffer', BytesField(
