@@ -2,9 +2,8 @@
 # Copyright: (c) 2019, Jordan Borean (@jborean93) <jborean93@gmail.com>
 # MIT License (see LICENSE or https://opensource.org/licenses/MIT)
 
-from __future__ import division
-
 import copy
+import datetime
 import math
 import struct
 import textwrap
@@ -20,22 +19,8 @@ from binascii import (
     hexlify,
 )
 
-from datetime import (
-    datetime,
-    timedelta,
-)
-
-from six import (
-    binary_type,
-    integer_types,
-    python_2_unicode_compatible,
-    text_type,
-    with_metaclass,
-)
-
 from smbprotocol._text import (
     to_bytes,
-    to_native,
     to_text,
 )
 
@@ -114,7 +99,7 @@ class Structure(object):
         )
         field_strings.append(hex_wrapper.fill(raw_hex))
 
-        string = "%s:\n%s" % (to_native(struct_name), '\n'.join([to_native(s) for s in field_strings]))
+        string = "%s:\n%s" % (to_text(struct_name), '\n'.join([to_text(s) for s in field_strings]))
 
         return string
 
@@ -144,9 +129,10 @@ class Structure(object):
         return data
 
     def unpack(self, data):
+        mem = memoryview(data)
         for key, field in self.fields.items():
-            data = field.unpack(data)
-        return data  # remaining data
+            mem = field.unpack(mem)
+        return bytes(mem)  # remaining data
 
     def _get_field(self, key):
         field = self.fields.get(key, None)
@@ -155,8 +141,7 @@ class Structure(object):
         return field
 
 
-@python_2_unicode_compatible
-class Field(with_metaclass(ABCMeta, object)):
+class Field(metaclass=ABCMeta):
 
     def __init__(self, little_endian=True, default=None, size=None):
         """
@@ -177,7 +162,7 @@ class Field(with_metaclass(ABCMeta, object)):
         field_type = self.__class__.__name__
         self.little_endian = little_endian
 
-        if not (size is None or isinstance(size, integer_types) or
+        if not (size is None or isinstance(size, int) or
                 isinstance(size, types.LambdaType)):
             raise InvalidFieldDefinition("%s size for field must be an int or "
                                          "None for a variable length"
@@ -227,6 +212,8 @@ class Field(with_metaclass(ABCMeta, object)):
         :param value: The value to be parsed and set, the allowed input types
             vary depending on the Field used
         """
+        if isinstance(value, memoryview):
+            value = bytes(value)
         parsed_value = self._parse_value(value)
         self.value = parsed_value
 
@@ -385,7 +372,7 @@ class IntField(Field):
             struct_string = "%s%s"\
                             % ("<" if self.little_endian else ">", format)
             int_value = struct.unpack(struct_string, value)[0]
-        elif isinstance(value, integer_types):
+        elif isinstance(value, int):
             int_value = value
         else:
             raise TypeError("Cannot parse value for field %s of type %s to "
@@ -414,7 +401,7 @@ class BytesField(Field):
             bytes_value = b""
         elif isinstance(value, types.LambdaType):
             bytes_value = value
-        elif isinstance(value, integer_types):
+        elif isinstance(value, int):
             format = self._get_struct_format(self.size)
             struct_string = "%s%s"\
                             % ("<" if self.little_endian else ">", format)
@@ -467,7 +454,7 @@ class ListField(Field):
         :param kwargs: Any other kwarg to be sent to Field()
         """
         if list_count is not None and not \
-                (isinstance(list_count, integer_types) or
+                (isinstance(list_count, int) or
                  isinstance(list_count, types.LambdaType)):
             raise InvalidFieldDefinition("ListField list_count must be an "
                                          "int, lambda, or None for a variable "
@@ -665,7 +652,6 @@ class StructureField(Field):
 class DateTimeField(Field):
 
     EPOCH_FILETIME = 116444736000000000  # epoch as a MS FILETIME int
-    HUNDREDS_NS = 10000000  # How many hundred nanoseconds in a second
 
     def __init__(self, size=None, **kwargs):
         """
@@ -690,20 +676,22 @@ class DateTimeField(Field):
         super(DateTimeField, self).__init__(size=8, **kwargs)
 
     def _pack_value(self, value):
-        epoch_seconds = self._seconds_since_epoch(value)
-        int_value = self.EPOCH_FILETIME + (epoch_seconds * self.HUNDREDS_NS)
-        int_value += value.microsecond * 10
+        utc_tz = datetime.timezone.utc
+        utc_dt = value.replace(tzinfo=value.tzinfo if value.tzinfo else utc_tz)
+        td = utc_dt.astimezone(utc_tz) - datetime.datetime(1970, 1, 1, 0, 0, 0, tzinfo=utc_tz)
+        epoch_time_ms = (td.microseconds + (td.seconds + td.days * 24 * 3600) * 10 ** 6)
+        ns100 = self.EPOCH_FILETIME + (epoch_time_ms * 10)
 
         format = self._get_struct_format(8)
         struct_string = "%s%s"\
                         % ("<" if self.little_endian else ">", format)
-        bytes_value = struct.pack(struct_string, int_value)
+        bytes_value = struct.pack(struct_string, ns100)
 
         return bytes_value
 
     def _parse_value(self, value):
         if value is None:
-            datetime_value = datetime.today()
+            datetime_value = datetime.datetime.today()
         elif isinstance(value, types.LambdaType):
             datetime_value = value
         elif isinstance(value, bytes):
@@ -712,12 +700,16 @@ class DateTimeField(Field):
                             % ("<" if self.little_endian else ">", format)
             int_value = struct.unpack(struct_string, value)[0]
             return self._parse_value(int_value)  # just parse the value again
-        elif isinstance(value, integer_types):
+        elif isinstance(value, int):
 
             time_microseconds = (value - self.EPOCH_FILETIME) // 10
-            datetime_value = datetime(1970, 1, 1) + \
-                timedelta(microseconds=time_microseconds)
-        elif isinstance(value, datetime):
+            try:
+                datetime_value = datetime.datetime(1970, 1, 1) + datetime.timedelta(microseconds=time_microseconds)
+            except OverflowError:
+                # This is unfortunately but 9999 is the max value a datetime can be so we just default to that
+                datetime_value = datetime.datetime.max
+
+        elif isinstance(value, datetime.datetime):
             datetime_value = value
         else:
             raise TypeError("Cannot parse value for field %s of type %s to a "
@@ -731,15 +723,6 @@ class DateTimeField(Field):
         datetime_value = self._get_calculated_value(self.value)
         return datetime_value.isoformat(' ')
 
-    def _seconds_since_epoch(self, datetime_value):
-        # total_seconds was not present in Python 2.6, this is suggested by
-        # Python docs as an alternative
-        # https://docs.python.org/2/library/datetime.html#datetime.timedelta.total_seconds
-        td = datetime_value - datetime.utcfromtimestamp(0)
-        seconds = (td.microseconds +
-                   (td.seconds + td.days * 24 * 3600) * 10 ** 6) / 10 ** 6
-        return int(seconds)
-
 
 class UuidField(Field):
 
@@ -769,7 +752,7 @@ class UuidField(Field):
             uuid_value = uuid.UUID(bytes=value)
         elif isinstance(value, bytes) and not self.little_endian:
             uuid_value = uuid.UUID(bytes_le=value)
-        elif isinstance(value, integer_types):
+        elif isinstance(value, int):
             uuid_value = uuid.UUID(int=value)
         elif isinstance(value, uuid.UUID):
             uuid_value = value
@@ -920,9 +903,9 @@ class TextField(BytesField):
     def _parse_value(self, value):
         if value is None:
             text_value = u""
-        elif isinstance(value, binary_type):
+        elif isinstance(value, bytes):
             text_value = to_text(value, encoding=self.encoding)
-        elif isinstance(value, text_type):
+        elif isinstance(value, str):
             text_value = value
         elif isinstance(value, types.LambdaType):
             text_value = value
@@ -944,12 +927,11 @@ class TextField(BytesField):
         return self.value
 
     def unpack(self, data):
-        import base64
         null_byte = u"\x00".encode(self.encoding)
 
         if self.null_terminated and self.size is None:
             # If the size wasn't specified and the string contains a null terminator, cut off the bytes there.
-            null_index = data.find(null_byte)
+            null_index = bytes(data).find(null_byte)
             if null_index != -1:
                 # In the case of UTF-16 we might be the index in the middle of a char, make sure we round up and get
                 # the boundary of each char.
@@ -960,7 +942,7 @@ class TextField(BytesField):
 
         size = self._get_calculated_size(self.size, data)
         raw_value = data[0:size]
-        if self.null_terminated and raw_value.endswith(null_byte):
+        if self.null_terminated and bytes(raw_value).endswith(null_byte):
             raw_value = raw_value[:len(null_byte) * -1]
 
         self.set_value(raw_value)
